@@ -399,8 +399,11 @@ class Py24POP3SSLinitMixIn(object):
 
             # Ensure cert is for server we're connecting to
             if ssl and self.conf['ca_certs']:
-                ssl_match_hostname(self.conn.sslobj.getpeercert(),
-                               self.conf['server'])
+                ssl_match_hostname(
+                    self.conn.sslobj.getpeercert(),
+                    self.conf.get('ssl_cert_hostname', None) 
+                        or self.conf['server']
+                )
 
             if ssl_fingerprints:
                 if not actual_hash:
@@ -634,8 +637,11 @@ class IMAPSSLinitMixIn(object):
 
             # Ensure cert is for server we're connecting to
             if ssl and self.conf['ca_certs']:
-                ssl_match_hostname(self.conn.ssl().getpeercert(),
-                               self.conf['server'])
+                ssl_match_hostname(
+                    self.conn.ssl().getpeercert(),
+                    self.conf.get('ssl_cert_hostname', None) 
+                        or self.conf['server']
+                )
 
             if ssl_fingerprints:
                 if not actual_hash:
@@ -1110,10 +1116,28 @@ class POP3RetrieverBase(RetrieverSkeleton):
         self.mailboxes = (None, )
         # Handle password
         if self.conf.get('password', None) is None:
-            self.conf['password'] = get_password(
-                self, self.conf['username'], self.conf['server'],
-                self.received_with, self.log
-            )
+            if self.conf.get('password_command', None):
+                # Retrieve from an arbitrary external command
+                command = self.conf['password_command'][0]
+                args = self.conf['password_command'][1:]
+                (rc, stdout, stderr) = run_command(command, args)
+                if stderr:
+                    self.log.warn(
+                        'External password program "%s" wrote to stderr: %s',
+                        command, stderr
+                    )
+                if rc:
+                    # program exited nonzero
+                    raise getmailOperationError(
+                        'External password program error (exited %d)' % rc
+                    )
+                else:
+                    self.conf['password'] = stdout
+            else:
+                self.conf['password'] = get_password(
+                    self, self.conf['username'], self.conf['server'],
+                    self.received_with, self.log
+                )
         RetrieverSkeleton.initialize(self, options)
         try:
             self._connect()
@@ -1183,7 +1207,8 @@ class MultidropPOP3RetrieverBase(POP3RetrieverBase):
         if self.envrecipname in NOT_ENVELOPE_RECIPIENT_HEADERS:
             raise getmailConfigurationError(
                 'the %s header field does not record the envelope '
-                'recipient address'
+                    'recipient address'
+                % self.envrecipname
             )
         self.envrecipnum = 0
         try:
@@ -1588,7 +1613,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             self.log.warning('Could not fetch google imap extensions: %s' % o)
             return {}
 
-        if not response:
+        if not response or not response[0]:
             return {}
             
         ext = re.search(
@@ -1633,10 +1658,28 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         # Handle password
         if (self.conf.get('password', None) is None
                 and not (HAVE_KERBEROS_GSS and self.conf['use_kerberos'])):
-            self.conf['password'] = get_password(
-                self, self.conf['username'], self.conf['server'], 
-                self.received_with, self.log
-            )
+            if self.conf['password_command']:
+                # Retrieve from an arbitrary external command
+                command = self.conf['password_command'][0]
+                args = self.conf['password_command'][1:]
+                (rc, stdout, stderr) = run_command(command, args)
+                if stderr:
+                    self.log.warn(
+                        'External password program "%s" wrote to stderr: %s',
+                        command, stderr
+                    )
+                if rc:
+                    # program exited nonzero
+                    raise getmailOperationError(
+                        'External password program error (exited %d)' % rc
+                    )
+                else:
+                    self.conf['password'] = stdout
+            else:
+                self.conf['password'] = get_password(
+                    self, self.conf['username'], self.conf['server'], 
+                    self.received_with, self.log
+                )
             
         RetrieverSkeleton.initialize(self, options)
         try:
@@ -1814,7 +1857,8 @@ class MultidropIMAPRetrieverBase(IMAPRetrieverBase):
         if self.envrecipname in NOT_ENVELOPE_RECIPIENT_HEADERS:
             raise getmailConfigurationError(
                 'the %s header field does not record the envelope recipient '
-                'address'
+                    'address'
+                % self.envrecipname
             )
         self.envrecipnum = 0
         try:
