@@ -62,6 +62,7 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
     )
@@ -95,6 +96,7 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
@@ -103,6 +105,7 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
         ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
         ConfString(name='ssl_version', required=False, default=None),
         ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -171,6 +174,7 @@ class BrokenUIDLPOP3Retriever(BrokenUIDLPOP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
     )
     received_with = 'POP3'
@@ -201,6 +205,7 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -208,6 +213,7 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
         ConfString(name='ssl_version', required=False, default=None),
         ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_with = 'POP3-SSL'
 
@@ -237,6 +243,7 @@ class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
@@ -270,6 +277,7 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfFile(name='keyfile', required=False, default=None),
@@ -278,6 +286,7 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
         ConfString(name='ssl_version', required=False, default=None),
         ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -316,6 +325,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         # Demon apparently doesn't support APOP
         ConfBool(name='use_apop', required=False, default=False),
     )
@@ -368,6 +378,7 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
@@ -409,6 +420,7 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
@@ -423,6 +435,7 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -454,6 +467,7 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
@@ -496,6 +510,7 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
@@ -511,6 +526,7 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
         ConfString(name='envelope_recipient'),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
